// BufferConverter.cpp: implementation of the CBufferConverter class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "TopfDebug.h"
#include "BufferConverter.h"

#include "common_definitions.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CBufferConverter::CBufferConverter()
{
	m_dwarrPC.RemoveAll();
	m_dwarrTopfield.RemoveAll();
}

CBufferConverter::~CBufferConverter()
{

}



LONG CBufferConverter::GetTopfieldPointer(LONG pc)
{
	for (int loop = 0; loop < m_dwarrPC.GetSize(); loop++) {
		if (m_dwarrPC[loop] == pc) return m_dwarrTopfield[loop];
	}

	return 0L;
}


LONG CBufferConverter::GetPCPointer(LONG topfield)
{
	for (int loop = 0; loop < m_dwarrTopfield.GetSize(); loop++) {
		if (m_dwarrTopfield[loop] == topfield) return m_dwarrPC[loop];
	}

	return 0L;
}


void CBufferConverter::AddPointer(LONG pc, LONG topfield)
{
	m_dwarrPC.Add(pc);
	m_dwarrTopfield.Add(topfield);
}


void CBufferConverter::RemoveTopfieldPointer(LONG topfield)
{
	for (int loop = 0; loop < m_dwarrTopfield.GetSize(); loop++) {
		if (m_dwarrTopfield[loop] == topfield) {
			m_dwarrTopfield.RemoveAt(loop);
			m_dwarrPC.RemoveAt(loop);
		}
	}
}


void CBufferConverter::RemovePCPointer(LONG pc)
{
	for (int loop = 0; loop < m_dwarrPC.GetSize(); loop++) {
		if (m_dwarrPC[loop] == pc) {
			m_dwarrTopfield.RemoveAt(loop);
			m_dwarrPC.RemoveAt(loop);
		}
	}
}


void CBufferConverter::SwapWord(WORD *w)
{
	unsigned short a = *w;
	union {
		WORD l;
		unsigned char b[2];
	} c1, c2;

	c1.l = a;
	c2.b[0] = c1.b[1];
	c2.b[1] = c1.b[0];
	a = c2.l;

	*w = a;
}

void CBufferConverter::SwapLong(LONG *w)
{
	union {
		LONG l;
		unsigned char b[4];
	} c1, c2;

	c1.l = *w;
	c2.b[0] = c1.b[3];
	c2.b[1] = c1.b[2];
	c2.b[2] = c1.b[1];
	c2.b[3] = c1.b[0];
	*w = c2.l;
}

void CBufferConverter::SwapULong64(ulong64 *w)
{
	union {
		ulong64 l;
		unsigned char b[8];
	} c1, c2;

	c1.l = *w;
	c2.b[0] = c1.b[7];
	c2.b[1] = c1.b[6];
	c2.b[2] = c1.b[5];
	c2.b[3] = c1.b[4];
	c2.b[4] = c1.b[3];
	c2.b[5] = c1.b[2];
	c2.b[6] = c1.b[1];
	c2.b[7] = c1.b[0];
	*w = c2.l;
}

//	void *eAddr;
//    void *oAddr;
//	word width;
//	word height;
//    word bytePerPixel;
//	word dataFormat;
//    word reserved[4];

void CBufferConverter::Convert_TYPE_OsdBaseInfo(TYPE_OsdBaseInfo *c)
{
	SwapLong((LONG*)&(c->eAddr));
	SwapLong((LONG*)&(c->oAddr));
	SwapWord((WORD*)&(c->width));
	SwapWord((WORD*)&(c->height));
	SwapWord((WORD*)&(c->bytePerPixel));
	SwapWord((WORD*)&(c->dataFormat));
	for (int loop = 0; loop < 4; loop++)
		SwapWord((WORD*)&(c->reserved[loop]));
}


//	byte 	version;
//	byte 	reserved;
//	byte 	dataFormat;
//	byte 	compessMethod;
//	void* 	data;
//	dword 	dataSize;
//	word 	width;
//	word 	height;

void CBufferConverter::Convert_TYPE_GrData(TYPE_GrData *c)
{
	SwapLong((LONG*)&(c->data));
	SwapLong((LONG*)&(c->dataSize));
	SwapWord((WORD*)&(c->width));
	SwapWord((WORD*)&(c->height));
}


//	char    satName[ MAX_SatName ];
//	char    chName[ MAX_SvcName ];
//	word	flag;
//	byte	tunerNum : 2;
//	byte	polar : 1;
//	word	freq;
//	word	sr;
//	word	svcId;
//	word	pmtPid;
//	word	pcrPid;
//	word	videoPid;
//	word	audioPid;
//	byte	volume;
//	byte	soundMode;
void CBufferConverter::Convert_TYPE_TapChInfo(TYPE_TapChInfo *c)
{
	SwapWord((WORD*)&(c->flag));
	SwapWord((WORD*)&(c->freq));
	SwapWord((WORD*)&(c->sr));
	SwapWord((WORD*)&(c->svcId));
	SwapWord((WORD*)&(c->pmtPid));
	SwapWord((WORD*)&(c->pcrPid));
	SwapWord((WORD*)&(c->videoPid));
	SwapWord((WORD*)&(c->audioPid));
}


//	word	evtId;				// event ID
//	dword	startTime;			// start time((MJD << 16) | (Hour << 8) | Min)
//	dword	endTime;
//	word	duration;			// duration((Hour << 8) | Min)
//	byte	runningStatus;		// running status
//	char	eventName[128];
//	char	description[128];
//	byte	parentalRating; 	// parental Rating
void CBufferConverter::Convert_TYPE_TapEvent(TYPE_TapEvent *c)
{
	SwapLong((LONG*)&(c->startTime));
	SwapLong((LONG*)&(c->endTime));
	SwapWord((WORD*)&(c->duration));
	SwapWord((WORD*)&(c->evtId));
}


//	byte *yAddress;
//	byte *cAddress;
//	word width;
//	word height;
void CBufferConverter::Convert_TYPE_VideoBank(TYPE_VideoBank *c)
{
	SwapLong((LONG*)&(c->yAddress));
	SwapLong((LONG*)&(c->cAddress));
	SwapWord((WORD*)&(c->width));
	SwapWord((WORD*)&(c->height));
}


//	byte	isRec;		// 1 = Recording, 0 = VCR Timer
//	byte	tuner;		// 0 = Tuner1 , 1 = Tuner2
//	byte	svcType;
//	byte	reserved;
//	word	svcNum;
//	byte	reservationType;
//	byte	nameFix;	// 1 = fileName is fixed
//	word	duration;
//	dword	startTime;
//	char	fileName[100];
void CBufferConverter::Convert_TYPE_TimerInfo(TYPE_TimerInfo *c)
{
	SwapLong((LONG*)&(c->startTime));
	SwapWord((WORD*)&(c->duration));
	SwapWord((WORD*)&(c->svcNum));
}


//	dword	attr:8;
//	dword	mjd:16;
//	dword	hour:8;
//	
//	dword	min :8;
//	dword	sec :8;
//	dword	localOffset:16;
//	
//	dword	startCluster;
//	dword	totalCluster;
//	dword	unusedByte;
//	char	name[ TS_FILE_NAME_SIZE ];
//    byte    skip;
//    byte    crypt;
//    byte	playLst;
//    byte    reserved[5];
//
//	void	*handle;
//	ulong64	size;
//	dword 	currentPos;
void CBufferConverter::Convert_TYPE_File(TYPE_File *c)
{
	CHAR *test = (CHAR*)c;
	SwapLong((LONG*)test);
	test = test + 4L;
	SwapLong((LONG*)test);
	SwapLong((LONG*)&(c->startCluster));
	SwapLong((LONG*)&(c->totalCluster));
	SwapLong((LONG*)&(c->unusedByte));
	SwapLong((LONG*)&(c->handle));
	SwapLong((LONG*)&(c->currentPos));
	SwapULong64((ulong64 *)&(c->size));
}


//	word titleBack;
//	word titleText;
//	word bodyBack;
//	word bodyText;	
//	word border;
//	word shadow;
//	word dark;
//	word light;
//	word check;
void CBufferConverter::Convert_TYPE_WindowColor(TYPE_WindowColor *c)
{
	SwapWord((WORD*)&(c->titleBack));
	SwapWord((WORD*)&(c->titleText));
	SwapWord((WORD*)&(c->bodyBack));
	SwapWord((WORD*)&(c->bodyText));
	SwapWord((WORD*)&(c->border));
	SwapWord((WORD*)&(c->shadow));
	SwapWord((WORD*)&(c->dark));
	SwapWord((WORD*)&(c->light));
	SwapWord((WORD*)&(c->check));
}


//	word  rgn;
//	dword x;
//	dword y;
//	dword w;
//	dword h;
//	dword titleX;
//	dword titleY;
//	dword titleW;
//	dword titleH;
//	dword bodyX;
//	dword bodyY;
//	dword bodyW;
//	dword bodyH;
//	TYPE_WindowColor color;
//	void  *saveBuf;
//	byte  save;
void CBufferConverter::Convert_TYPE_Win(TYPE_Win *c)
{
	SwapWord((WORD*)&(c->rgn));
	SwapLong((LONG*)&(c->x));
	SwapLong((LONG*)&(c->y));
	SwapLong((LONG*)&(c->w));
	SwapLong((LONG*)&(c->h));
	SwapLong((LONG*)&(c->titleX));
	SwapLong((LONG*)&(c->titleY));
	SwapLong((LONG*)&(c->titleW));
	SwapLong((LONG*)&(c->titleH));
	SwapLong((LONG*)&(c->bodyX));
	SwapLong((LONG*)&(c->bodyY));
	SwapLong((LONG*)&(c->bodyW));
	SwapLong((LONG*)&(c->bodyH));
	SwapWord((WORD*)&(c->saveBuf));
	Convert_TYPE_WindowColor(&(c->color));
}


//	TYPE_Win win;
//	char	*itemBuf;
//	char	*item[MAX_ListWindowItem];	//[MAX_ListWindowItemLen];
//	byte	check[MAX_ListWindowItem];
//	word	idx[MAX_ListWindowItem];
//	long	itemNum;
//	long	startPos;
//	long	currentPos;
//	long	maxItemView;
//	byte	fntType;
//	byte	fntSize;
//	byte    bScr;
//	byte	onlyEng;
void CBufferConverter::Convert_TYPE_Window(TYPE_Window *c)
{
	int loop;
	Convert_TYPE_Win(&(c->win));

	for (loop = 0; loop < MAX_ListWindowItem; loop++) {
		SwapWord((WORD*)&(c->idx[loop]));
		SwapLong((LONG*)&(c->item[loop])); // swap the pointer to the string
	}


	SwapLong((LONG*)&(c->itemNum));
	SwapLong((LONG*)&(c->startPos));
	SwapLong((LONG*)&(c->currentPos));

	SwapLong((LONG*)&(c->maxItemView));
	SwapLong((LONG*)&(c->itemBuf));
}


